@@ -407,21 +407,28 @@ class Script(default.Script):
 
         if self.flatReviewContext \
            or not self.isBrailleBeginningShowing() \
-           or self.utilities.isSpreadSheetCell(orca_state.locusOfFocus):
+           or self.utilities.isSpreadSheetCell(orca_state.locusOfFocus) \
+           or not self.utilities.isTextArea(orca_state.locusOfFocus):
             return default.Script.panBrailleLeft(self, inputEvent, panAmount)
 
+        text = orca_state.locusOfFocus.queryText()
+        string, startOffset, endOffset = text.getTextAtOffset(
+            text.caretOffset, pyatspi.TEXT_BOUNDARY_LINE_START)
+        if 0 &lt; startOffset:
+            text.setCaretOffset(startOffset-1)
+            return True
+
         obj = self.utilities.findPreviousObject(orca_state.locusOfFocus)
-        orca.setLocusOfFocus(None, obj, notifyScript=False)
-        self.updateBraille(obj)
-
-        # Hack: When panning to the left in a document, we want to start at
-        # the right/bottom of each new object. For now, we'll pan there.
-        # When time permits, we'll give our braille code some smarts.
-        while self.panBrailleInDirection(panToLeft=False):
+        try:
+            text = obj.queryText()
+        except:
             pass
-        self.refreshBraille(False)
+        else:
+            orca.setLocusOfFocus(None, obj, notifyScript=False)
+            text.setCaretOffset(text.characterCount)
+            return True
 
-        return True
+        return default.Script.panBrailleLeft(self, inputEvent, panAmount)
 
     def panBrailleRight(self, inputEvent=None, panAmount=0):
         """In document content, we want to use the panning keys to browse the
@@ -430,21 +437,28 @@ class Script(default.Script):
 
         if self.flatReviewContext \
            or not self.isBrailleEndShowing() \
-           or self.utilities.isSpreadSheetCell(orca_state.locusOfFocus):
+           or self.utilities.isSpreadSheetCell(orca_state.locusOfFocus) \
+           or not self.utilities.isTextArea(orca_state.locusOfFocus):
             return default.Script.panBrailleRight(self, inputEvent, panAmount)
 
+        text = orca_state.locusOfFocus.queryText()
+        string, startOffset, endOffset = text.getTextAtOffset(
+            text.caretOffset, pyatspi.TEXT_BOUNDARY_LINE_START)
+        if endOffset &lt; text.characterCount:
+            text.setCaretOffset(endOffset)
+            return True
+
         obj = self.utilities.findNextObject(orca_state.locusOfFocus)
-        orca.setLocusOfFocus(None, obj, notifyScript=False)
-        self.updateBraille(obj)
-
-        # Hack: When panning to the right in a document, we want to start at
-        # the left/top of each new object. For now, we'll pan there. When time
-        # permits, we'll give our braille code some smarts.
-        while self.panBrailleInDirection(panToLeft=True):
+        try:
+            text = obj.queryText()
+        except:
             pass
-        self.refreshBraille(False)
+        else:
+            orca.setLocusOfFocus(None, obj, notifyScript=False)
+            text.setCaretOffset(0)
+            return True
 
-        return True
+        return default.Script.panBrailleRight(self, inputEvent, panAmount)
 
     def presentInputLine(self, inputEvent):
         """Presents the contents of the spread sheet input line (assuming we
@@ -892,6 +906,30 @@ class Script(default.Script):
 
         default.Script.onChildrenChanged(self, event)
 
+    def onFocus(self, event):
+        """Callback for focus: accessibility events."""
+
+        # NOTE: This event type is deprecated and Orca should no longer use it.
+        # This callback remains just to handle bugs in applications and toolkits
+        # during the remainder of the unstable (3.11) development cycle.
+
+        role = event.source.getRole()
+
+        # This seems to be something we inherit from Gtk+
+        if role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_PASSWORD_TEXT]:
+            orca.setLocusOfFocus(event, event.source)
+            return
+
+        # Ditto.
+        if role == pyatspi.ROLE_PUSH_BUTTON:
+            orca.setLocusOfFocus(event, event.source)
+            return
+
+        # Ditto.
+        if role == pyatspi.ROLE_COMBO_BOX:
+            orca.setLocusOfFocus(event, event.source)
+            return
+
     def onFocusedChanged(self, event):
         """Callback for object:state-changed:focused accessibility events."""
 
@@ -914,6 +952,10 @@ class Script(default.Script):
         if start != end:
             return
 
+        # We should present this in response to active-descendant-changed events
+        if event.source.getState().contains(pyatspi.STATE_MANAGES_DESCENDANTS):
+            return
+
         default.Script.onFocusedChanged(self, event)
 
     def onCaretMoved(self, event):
@@ -930,7 +972,15 @@ class Script(default.Script):
             return
 
         if self.utilities.isCellBeingEdited(event.source):
-            orca.setLocusOfFocus(event, event.source.parent)
+            orca.setLocusOfFocus(event, event.source.parent, False)
+
+        if not orca_state.locusOfFocus:
+            default.Script.onCaretMoved(self, event)
+            return
+
+        if orca_state.locusOfFocus.getRole() == pyatspi.ROLE_TABLE_CELL:
+            default.Script.onCaretMoved(self, event)
+            return
 
         # The lists and combo boxes in the Formatting toolbar emit
         # object:active-descendant-changed events which cause us
@@ -954,7 +1004,9 @@ class Script(default.Script):
 
         obj = event.source
         role = obj.getRole()
-        if not role in [pyatspi.ROLE_TOGGLE_BUTTON, pyatspi.ROLE_PUSH_BUTTON]:
+        parentRole = obj.parent.getRole()
+        if not role in [pyatspi.ROLE_TOGGLE_BUTTON, pyatspi.ROLE_PUSH_BUTTON] \
+           or not parentRole == pyatspi.ROLE_TOOL_BAR:
             default.Script.onCheckedChanged(self, event)
             return
  
@@ -974,6 +1026,12 @@ class Script(default.Script):
         if weToggledIt:
             speech.speak(self.speechGenerator.generateSpeech(obj))
 
+    def onRowReordered(self, event):
+        """Callback for object:row-reordered accessibility events."""
+
+        # We're seeing a crazy ton of these emitted bogusly.
+        pass
+
     def onTextAttributesChanged(self, event):
         """Callback for object:text-attributes-changed accessibility events."""
 
